import {
  Button, Form, LoadingService, Modal, ModalService, SelectMultiple, Text,
} from '@zeedhi/common';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import {
  Instances, Row, InstanceResult, ParamsBlockStore, ValuesSelectStore, SelectMultiplesStore,
} from '@/types/InterfaceStoreBlockController';
import { alertService } from '@/helpers/AlertService';

export class StoreBlockController {
  private storeBlockModal!: Modal;

  private selectedStore: SelectMultiplesStore[] = [];

  private keyPressed: string = '';

  public async createStoreBlockModal(): Promise<void> {
    this.storeBlockModal = await ModalService.createFromJson('store-block-modal', true);
  }

  public destroyStoreBlockModal(): void {
    this.storeBlockModal.destroy();
  }

  public openStoreBlockModal(): void {
    this.storeBlockModal.show();
  }

  public async closeStoreBlockModal(): Promise<void> {
    const form = await this.getInstance<Form>('store-block-form');
    form.clearForm();
    this.storeBlockModal.hide();
  }

  public async mounted(): Promise<void> {
    await this.store();
    this.storeLineEvent();
  }

  public grid(): TekTreeGrid {
    return Metadata.getInstance<TekTreeGrid>('store-block-grid');
  }

  public async getInstance<T extends Instances>(name: string): Promise<T> {
    const instance: T = await new Promise((resolve) => {
      setTimeout(() => {
        resolve(Metadata.getInstances(name)[0]);
      }, 0);
    });
    return instance;
  }

  public async instances(): Promise<InstanceResult> {
    const select = await this.getInstance<SelectMultiple>('store');
    const selectWeek = await this.getInstance<SelectMultiple>('week');
    const button = await this.getInstance<Button>('store-block-save');
    const textCdFilial = await this.getInstance<Text>('FILIAL');
    const block = await this.getInstance<Text>('BLOCK');
    return {
      select,
      selectWeek,
      button,
      textCdFilial,
      block,
    };
  }

  public async store(): Promise<void> {
    try {
      const grid = this.grid();
      LoadingService.show();
      const { data: { data, message, error } } = await Http.get('/branch/storeBlock');

      if (error) throw new Error(message);

      grid.datasource.updateData(data);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public selectMultipleStores(component: any, row: any): void {
    const children = row.tree__children.some((item: any) => item.tree__children.length > 0);

    if (this.keyPressed === 'control' && children) {
      component.selectRow(row, true);
      component.selectChildren(row);

      row.tree__children.forEach((firstLevel: any) => {
        firstLevel.tree__children.forEach((element: any) => {
          this.selectedStore.push({
            CDFILIAL: element.FILIAL,
            CDLOJA: element.CDFILIALGRID,
            NRDIASEMANABLOQ: element.NRDIASEMANABLOQ,
          });
        });
      });
    } else if (this.keyPressed === 'shift') {
      component.selectRow(row, false);
      component.unselectChildren(row);
      this.selectedStore = [];
    }
  }

  public async openStoreModal({ component, row }: IEventParam<TekTreeGrid>): Promise<void> {
    this.selectMultipleStores(component, row);
    if (row.BLOCK === 'ISTRUE') {
      const selectedStore = {
        CDFILIAL: row.FILIAL,
        CDLOJA: row.CDFILIALGRID,
        NRDIASEMANABLOQ: row.NRDIASEMANABLOQ,
      };
      const storeIndex = this.selectedStore.findIndex((item) => item.CDFILIAL === selectedStore.CDFILIAL
        && item.NRDIASEMANABLOQ === selectedStore.NRDIASEMANABLOQ);

      if (storeIndex > -1) {
        component.selectRow(row, false);
        this.selectedStore.splice(storeIndex, 1);
      } else {
        component.selectRow(row, true);
        this.selectedStore.push(selectedStore);
      }
    } else if (!row.DTHRBLOQUEIO && row.tree__level !== 1) {
      this.openStoreBlockModal();
      await this.modalLockOrUnlock(row);
    }
  }

  public async modalLockOrUnlock(row: Row): Promise<void> {
    const {
      select, selectWeek, button, textCdFilial, block,
    } = await this.instances();
    block.text = row.BLOCK;
    textCdFilial.text = row.FILIAL;

    const params = {
      NMLOJA: row.NMLOJA,
      CDFILIALGRID: row.CDFILIALGRID,
    };

    if (row.tree__level === 2) {
      this.setWeekValue(row, selectWeek);
      this.alterValueSelectAndButtonLockOrUnlock(select, button, 'Bloquear', params);
    } else {
      selectWeek.disabled = false;
      this.alterValueSelectAndButtonLockOrUnlock(select, button, 'Bloquear', params);
    }
  }

  public alterValueSelectAndButtonLockOrUnlock(select: SelectMultiple,
    button: Button, label: string, values: ValuesSelectStore): void {
    select.updateData([values]);
    select.selectAllItems();
    select.disabled = true;

    button.label = label;
  }

  public setWeekValue(row: Row, select: SelectMultiple): void {
    const { tree__children: children } = row;
    const { data } = select.datasource;
    const blockedDays = children.map((item: any) => item.NRDIASEMANABLOQ);
    const daysIncluded = data.filter((item: any) => blockedDays.includes(item.value));

    select.disabled = false;
    if (blockedDays[0] === 'T') {
      select.selectAllItems();
    } else {
      select.setValue(daysIncluded);
    }
  }

  public async isValidFormBlockStore(): Promise<boolean> {
    const form = await this.getInstance<Form>('store-block-form');
    const valid = form.validate();
    return valid;
  }

  public async paramsBlockStoreModal(): Promise<ParamsBlockStore> {
    const form = await this.getInstance<Form>('store-block-form');
    const block = await this.getInstance<Text>('BLOCK');
    const cdOperador = localStorage.getItem('ID');
    const { text } = await this.getInstance<Text>('FILIAL');
    const { value } = form;
    return {
      CDFILIAL: text,
      CDLOJA: value.store,
      CDOPERADOR: cdOperador,
      NRDIASEMANABLOQ: value.week.length === 7 ? ['T'] : value.week,
      BLOCK: block.text,
    };
  }

  public async blockStore(): Promise<any> {
    const isValid = await this.isValidFormBlockStore();
    if (!isValid) return;
    const params = await this.paramsBlockStoreModal();
    await this.block(params);
  }

  public async block(params: ParamsBlockStore): Promise<void> {
    try {
      LoadingService.show();

      const { data: { error, message } } = await Http.post('/branch/blockStores', params);

      if (error) throw new Error(message || 'Erro na solicitação da API');

      alertService('success', message, 'green');
      this.closeStoreBlockModal();
      await this.store();
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public async unlock(): Promise<void> {
    try {
      LoadingService.show();
      const params = this.selectedStore;

      const { data: { error, message } } = await Http.post('/branch/unlockStores', params);

      if (error) throw new Error(message || 'Erro na solicitação da API');

      alertService('success', message, 'green');
      await this.store();
      const active = document.querySelectorAll('#store-block-grid table tbody tr.active')[0];
      active.classList.remove('active');
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public storeLineEvent(): void {
    document.addEventListener('keydown', (event) => {
      if (event.key === 'Control') {
        this.keyPressed = 'control';
      } else if (event.key === 'Shift') {
        this.keyPressed = 'shift';
      }
    });
    document.addEventListener('keyup', (event) => {
      if (event.key === 'Control') {
        this.keyPressed = '';
      } else if (event.key === 'Shift') {
        this.keyPressed = '';
      }
    });
  }
}
